{
  Unit: PMG.PAS
  Author: Bostjan Gorisek (Gury)
          Tebe - Core and supporting routines (Poke2, Peek2, Fillchar2)
  Release date: 6.10.2015
  Version: 1.0 R1 Beta
}

const
  _PM_MAX = 14;         // Number of player data values
  _PM_NORMAL_SIZE = 0;  // PM normal size
  _PM_DOUBLE_SIZE = 1;  // PM double size
  _PM_QUAD_SIZE   = 3;  // PM quadruple size
  _PM_SHOW_ON     = 3;  // Show PM graphics
  _PM_SHOW_OFF    = 0;  // Hide/reset PM graphics
  _PM_DOUBLE_RES  = 1;  // PM double-line resolution
  _PM_SINGLE_RES  = 2;  // PM single-line resolution

var
  // Player data variables
  p0Data, p1Data, p2Data, p3Data : array [0.._PM_MAX] of byte;
  // PM graphics supporting variables  
  pm_mem : word;
  pm_offset : word = 512;
  pm_top : byte = 4;  
  pm_size : word = 128;

//-----------------------------------------------------------------------------
// Procedure  : Poke2
// Description: Poke to memory location (1 byte)
// Parameters : a (word) - memory location number
//              value (byte) - new value for selected memory location 
//-----------------------------------------------------------------------------
procedure Poke2(a : word; value : byte);
begin
asm
{	mwa a ztmp
	ldy #0
	mva value (ztmp),y
};
end;

//-----------------------------------------------------------------------------
// Function   : Peek2
// Description: Read memory location (1 byte)
// Parameters : a (word) - memory location number
// Returns    : (byte) Memory location found
//-----------------------------------------------------------------------------
function Peek2(a : word) : byte;
begin
asm
{	mwa a ztmp
	ldy #0
	mva (ztmp),y Result
};
end;

//-----------------------------------------------------------------------------
// Procedure  : Fillchar2
// Description: Fills the memory starting at A with Count bytes or characters with value equal to Value
// Parameters : a (pointer) - memory data pointed to
//              count (word) - number of bytes to fill
//              value (byte) - new value for selected memory pointed to 
//-----------------------------------------------------------------------------
procedure Fillchar2(a : pointer; count : word; value : byte);
begin
asm
{	stx @sp

	mwa a dst

	ldx #0
	ldy #0
	lda value

loop	cpy count
	bne @+
	cpx count+1
	beq stop

@	sta $ffff,y
dst	equ *-2

	iny
	bne @+
	inc dst+1
	inx
@
	bne loop

stop	ldx @sp
};
end;

//-----------------------------------------------------------------------------
// Procedure  : Move2
// Description: Moves Count bytes from Source to Dest
// Parameters : source (pointer) - source data pointer
//              dest (pointer)  - destination data pointer
//              count (word) - count of bytes to move
//-----------------------------------------------------------------------------
procedure Move2(source, dest : pointer; count : word);
begin
asm
{
  @move source dest count count+1
};
end;

//-----------------------------------------------------------------------------
// Procedure  : SetPM
// Description: Set PM variables
// Parameters : res (byte) - PM graphics resolution type
//                           (_PM_SINGLE_RES, _PM_DOUBLE_RES)
//-----------------------------------------------------------------------------
procedure SetPM(res : byte);
begin
  pm_offset := 512 * res;
  pm_size := 128 * res;
  if res = _PM_SINGLE_RES then
    pm_top := 16    
  else
    pm_top := 8;
end;
 
//-----------------------------------------------------------------------------
// Procedure  : ClearPM
// Description: Clear player memory
//-----------------------------------------------------------------------------
procedure ClearPM;
begin
  fillchar2(pointer(pm_mem+pm_offset), pm_size*4, 0);
end;

//-----------------------------------------------------------------------------
// Procedure  : MovePM
// Description: Draw and move selected player
// Parameters : pm (byte) - selected player
//              x (word) - player horizontal coordinate
//              y (byte) - player vertical coordinate
//-----------------------------------------------------------------------------
procedure MovePM(pm : byte; x : word; y : byte);
begin
  // Erase top and bottom line of player data
  Poke2(pm_mem+pm_offset+pm_size*pm+y-1, 0);
  Poke2(pm_mem+pm_offset+pm_size*pm+y+_PM_MAX+1, 0);
  // Vertical position of player   
  if pm = 0 then begin
    Move2(p0Data, pointer(pm_mem+pm_offset+y), sizeof(p0Data));
  end else if pm = 1 then begin
    Move2(p1Data, pointer(pm_mem+pm_offset+pm_size*pm+y), sizeof(p1Data));
  end else if pm = 2 then begin
    Move2(p2Data, pointer(pm_mem+pm_offset+pm_size*pm+y), sizeof(p2Data));
  end else if pm = 3 then begin
    Move2(p3Data, pointer(pm_mem+pm_offset+pm_size*pm+y), sizeof(p3Data));
  end;       
  // Horizontal position of player
  Poke2(53248+pm,x);
end;

//-----------------------------------------------------------------------------
// Procedure  : ColorPM
// Description: Player color
// Parameters : pm (byte) - selected player
//              col (byte) - player color
//-----------------------------------------------------------------------------
procedure ColorPM(pm, col : byte);
begin
  Poke2(704+pm, col);
end;

//-----------------------------------------------------------------------------
// Procedure  : SizePM
// Description: Player size
// Parameters : pm (byte) - selected player
//              value (byte) - player size:
//                             _PM_NORMAL_SIZE (0 = normal)
//                             _PM_DOUBLE_SIZE (1 = double)
//                             _PM_QUAD_SIZE   (3 = quadruple)
//-----------------------------------------------------------------------------
procedure SizePM(pm, value : byte);
begin
  Poke2(53256+pm, value);
end;

//-----------------------------------------------------------------------------
// Procedure  : ShowPM
// Description: PM graphics visibility
// Parameters : show (byte) - selected player
//                            (_PM_SHOW_ON, _PM_SHOW_OFF)
//-----------------------------------------------------------------------------
procedure ShowPM(show : byte);
begin
  Poke2(53277, show);
end;

//-----------------------------------------------------------------------------
// Procedure  : InitPM
// Description: Initialize PM graphics
// Parameters : res (byte) - selected player
//                            (_PM_DOUBLE_RES, _PM_SINGLE_RES)
//-----------------------------------------------------------------------------
procedure InitPM(res : byte);
begin
  Poke2(53277, 0);
  pm_mem := Peek2(106) - pm_top;
  Poke2(54279, pm_mem);
  pm_mem := pm_mem * 256;
  if res = _PM_DOUBLE_RES then
    Poke2(559, 46)
  else begin
    Poke2(559, 62);
  end;    
  ClearPM;
end;
